import pygame
from quit import *
from game import *
from multiplayerscreen import *
from settingsscreen import *
from gui.dialog import *
from gui.image import *
from players import *

class GameScreen(Dialog):
	def __init__(self, surface, images, locations):
		self.images = images
		self.locations = locations
		Dialog.__init__ (self, surface)
	
	def createWidgets(self):
		# Set the background
		self.surface.blit(self.images['gamescreen']['background'],(0,0))
		
		self.wm.register(Button(self.images['gamescreen']['humanvscpu'], self.images['gamescreen']['humanvscpu'],
			(int(self.locations['gamescreen']['humanvscpu_x']),
                        int(self.locations['gamescreen']['humanvscpu_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.humanvscpu }
		))
		self.wm.register(Button(self.images['gamescreen']['multiplayer'], self.images['gamescreen']['multiplayer'],
			(int(self.locations['gamescreen']['multiplayer_x']),
                        int(self.locations['gamescreen']['multiplayer_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.multiplayergame }
		))
		self.wm.register(Button(self.images['gamescreen']['back'], self.images['gamescreen']['back'],
                        (int(self.locations['gamescreen']['back_x']),
                        int(self.locations['gamescreen']['back_y'])),
                        callbacks={widget.MOUSEBUTTONUP : self.back }
                ))

	def humanvscpu(self, trigger, event):
		player1 = StrategicMinMax(self.surface, self.images, self.locations, 2)
		player2 = Human(self.surface, self.images, self.locations, 2)
		
		game = Game(self.surface, self.images, self.locations, player1, player2)
		game.run()
		
		self.surface.blit(self.images['startscreen']['background'],(0,0))
		self.wm.paint(1,0)

	def multiplayergame(self, trigger, event):
		multiplayerscreen = MultiplayerScreen(self.surface, self.images, self.locations)
		multiplayerscreen.run()
		self.surface.blit(self.images['startscreen']['background'],(0,0))
                self.wm.paint(1,0)

	def back(self, trigger, event):
		self.state = 1
		return widget.DONE

	def loadPlayer(self, name):
                difficulty = 4 #int(self.settings.getGlobal()['game'][name+'_level'])

                if self.settings.getGlobal()['game'][name+'_type'] == 'human':
	                player = Human(self.screen, self.images, self.settings, difficulty)
                elif self.settings.getGlobal()['game'][name+'_type'] == 'strategic':
                        player = StrategicMinMax(self.screen, self.images, self.settings, difficulty)
		elif self.settings.getGlobal()['game'][name+'_type'] == 'random':
                        player = RandomAI(self.screen, self.images, self.settings, difficulty)
                elif self.settings.getGlobal()['game'][name+'_type'] == 'minmax':
			player = MinMax(self.screen, self.images, self.settings, difficulty)
                elif self.settings.getGlobal()['game'][name+'_type'] == 'weighted':
                        player = WeightedMinMax(self.screen, self.images, self.settings, difficulty)
                else:
			print "INI FILE CURRUPT: set a player type to a legal one"
		        print "Legal players include:"
		        print "human, strategic, random, minmax"
		        quit()

		return player



# Echo server program
import socket
import rules

class MultiPlayer:
	def __init__(self, surface, images, locations, connection, turn):
		self.connection = connection
		
		self.myturn = turn

	def doMove(self, board, player, event):
		# Transmit the last move first
		if self.myturn:
			# Add one to the last move before sending to make sure it
			# get's send: str(0) = ""
			self.connection.send(str(board.last_move+1))
			self.myturn = 0
		else:
			# Wait for a move to return
			try:
				# Convert the string to an int and substract the 1
				move = int(self.connection.recv(1024))-1
				self.myturn = 1
				return move
			except:
				return -1

	def gameOver(self, last_move):
		self.connection.send(str(last_move+1))
		self.connection.close()



###############################################################################################
# $Id: info_units.py,v 1.1.1.1 2002/02/19 10:16:10 slmm Exp $
###############################################################################################

import scenario
import properties
import organization
import widget
import unit
from button import Button
from label import Label
from widget_manager import WidgetManager

class InfoUnits:
    """
    This class is used as a dialog for showing the 'order of battle', i.e. which units participate
    in the battle from the rebel and union sides. The full organizations are shown along with all
    available stats for all units.
    """

    def __init__ (self):
        "Creates the dialog."

        # create a new surface matching the display
        self.surface = sdl.surface_new ( 0, properties.window_size_x, properties.window_size_y,
                                         properties.window_depth, (0,0,0,0) ).convert_display () 
        
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 16 )
        self.tinyfont  = pygame.font.Font  ( properties.tinyfont, 12 )


        # create the labels for the rebel and union units
#        rebellabels = self.createLabels ( organization.REBEL )
#        unionlabels = self.createLabels ( organization.UNION )

        # loop over all lables we got
#        for index in range ( len (rebellabels) ):
#            # get the current label
#            indent, labels = rebellabels[index]
#
#            # do we have one or more labels?
#            if len (labels) == 1:
#                # we have only one label, it is a highlevel organization of some kind. Get the label
#                label = labels [0]
#                
#                # blit the label
#                self.surface.blit ( label, (0, 0, label.get_width(), label.get_height()),
#                                    (50 + indent * 50, 100 + index * label.get_height(), label.get_width(),
#                                     label.get_height() ) )
#            else:
#                # it is a tuple of labels for a unit
#                name, type, men, guns = labels
#
#                # blit the labels
#                self.surface.blit ( name, (0, 0, name.get_width(), name.get_height()),
#                                    (50 + indent * 50, 100 + index * name.get_height(), name.get_width(),
#                                     name.get_height() ) )
#                self.surface.blit ( type, (0, 0, type.get_width(), type.get_height()),
#                                    (100 + indent * 50, 100 + index * type.get_height(), type.get_width(),
#                                     type.get_height() ) )
#                self.surface.blit ( men, (0, 0, men.get_width(), men.get_height()),
#                                    (150 + indent * 50, 100 + index * men.get_height(), men.get_width(),
#                                     men.get_height() ) )
#                self.surface.blit ( guns, (0, 0, guns.get_width(), guns.get_height()),
#                                    (200 + indent * 50, 100 + index * guns.get_height(), guns.get_width(),
#                                     guns.get_height() ) )
#
        # create the widget manager
        self.wm = WidgetManager (self.surface)

        # create all widgets
        self.createWidgets ()
            
                
        # blit out or full surface to the main surface
#        scenario.sdl.blit ( self.surface, (0, 0, properties.window_size_x, properties.window_size_y )
#                            (0, 0, properties.window_size_x, properties.window_size_y ) )

        # update the whole screen
        scenario.sdl.update_rect ( (0, 0, properties.window_size_x, properties.window_size_y ) )


    def createWidgets (self):
        "Creates all widgets for the dialog."
        # create the buttons for the next and previous buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-back-moff.png",
                                    properties.path_dialogs + "butt-back-mover.png",
                                    (850, 30 ), {widget.MOUSEBUTTONUP : self.prevScreen } ) )

        # labels
        self.wm.register ( Label (self.titlefont, "Scenario order of battle ", (10,10), color=(255, 0, 0)) )

        unionlabel = Label (self.smallfont, "Union brigades", (50, 80), color=(255, 255, 0) )
        self.wm.register ( unionlabel )

        # create the label summarizing the units of the union side
        unionlabel = self.createSummary ( organization.UNION, 200, 83 )
        self.wm.register ( unionlabel )
        
        # create labels for all union brigades and store the coordinate of the last used y-position
        lasty = self.createBrigadeLabels ( organization.UNION, 80 + unionlabel.getHeight () + 30 );

        
        rebellabel = Label (self.smallfont, "Rebel brigades", (50, lasty + 30), color=(255, 255, 0))
        self.wm.register ( rebellabel )

        # create the label summarizing the units of the rebel side
        rebellabel = self.createSummary ( organization.REBEL, 200, lasty + 30 + 3 )
        self.wm.register ( rebellabel )

        # create labels for all rebel brigades
        unionBrigades = self.createBrigadeLabels ( organization.REBEL,
                                                   lasty + 30 + rebellabel.getHeight () + 30 );


    def createBrigadeLabels (self, owner, starty):
        """Creates labels for all brigades owned by 'owner'."""
        index = starty

        # loop over all brigades owned by the correct player
        for brigade in scenario.brigades [owner].values ():

            # render a label and register it
            label =  Label (self.smallfont, brigade.getName (), (150, index),
                            color=(255, 0, 255))

            # register label so that it gets managed
            self.wm.register ( label )

            # also create a label with info about the brigade
            # increment the index
            index += label.getHeight () + 20

        # we're done, return the last index we used
        return index
    

    def createSummary (self, owner, x, y):
        "Creates a summary label for all the units of the specified owner."

        companies = []
        
        # first get all companies of all the brigades
        for brigade in scenario.brigades [ owner ].values ():
            companies = companies + brigade.getCompanies ()

        # now set some counters
        counts = { unit.INFANTRY: 0,
                   unit.CAVALRY: 0,
                   unit.ARTILLERY: 0 }
        guns = 0

        # loop over all companies
        for company in companies:
            # add the men to the proper counter
            counts [company.getType ()] += company.getMen ()
            guns += company.getGuns ()

        # now create a suitable label
        labeltext  = "infantry: " + str (counts[unit.INFANTRY]) + ", cavalry: " + str (counts[unit.CAVALRY])
        labeltext += ", artillery (men/guns): " + str (counts[unit.ARTILLERY]) + "/" + str (guns)

        # create and return the label
        return Label ( self.tinyfont, labeltext, (x,y), color=(255, 255, 255))
        

    def run (self):
        """Executes the dialog and runs its internal loop until a key is pressed. When that happens
        this dialog is terminated and the method returns."""

        # loop forever
        while 1:
            # get next event
            event = sdl.events_wait ()

            # see wether the widget manager wants to handle it
            if event != -1:
                # handle event and get the return code that tells us wehter it was handled or not
                returncode = self.wm.handle ( event )

                # is the event loop done?
                if returncode == widget.DONE:
                    return


    def prevScreen (self, event):
        """Callback triggered when the user clicks the 'Previous' button. This method simply cancels
        the event loop for this dialog's widget manager. Basically this dialog quits."""
        # we're done
        return widget.DONE
        # loop forever
        while 1:
            # get next event
            event = sdl.events_wait ()

            # see wether the widegt manager wants to handle it
            if event != -1 and self.wm.handle ( event ):
                # it's handled, nothing to see here folks
                continue

            # did we gert anything?
            if event != -1 and event.get_type () == sdl.KEYDOWN:
                # we're done here, go away
                return



#    def createLabels (self, owner):
#        """Creates the labels for all organizations. The returned data is a list with pairs of the
#        type (label,indent). The label is the surface to be blitted and the indent is the
#        indentation this label should have. A more indented label is a part of a larger organization
#        (which is less indented)."""
#        labels = []
#
#        # loop over all brigades owned by the correct player
#        for brigade in scenario.brigades [owner].values ():
#
#            # render a label
#            labels.append ( self.renderLabel ( self.tinyfont, brigade.getName (), (255,0,255), 0 ))
#
#            # loop over all regiments
#            for regiment in brigade.getRegiments ():
#                # render a label
#                labels.append ( self.renderLabel ( self.tinyfont, regiment.getName (), (0,255,255), 1 ))
#
#                # loop over all battallions
#                for battallion in regiment.getBattallions ():
#                    # render a label
#                    labels.append ( self.renderLabel ( self.tinyfont, battallion.getName (),
#                                                           (128,128,128), 2 ))
#
#                    # loop over the companies
#                    for company in battallion.getCompanies ():
#                        # render a label
#                        labels.append ( self.renderUnitLabels ( self.tinyfont, company, (255,255,0), 3 ))
#
#                # and companies too
#                for company in regiment.getCompanies ():
#                    # render a label
#                    labels.append ( self.renderUnitLabels ( self.tinyfont, company, (255,0,255), 3 ))
#
#        return labels
#            
#
#    def renderLabel (self, font, text, color, indent):
#        """Help method for rendering one single label. Returns a pair with (label, indent)."""
#        return ( indent, (font.render ( text, color, (0,0,0) ), ) )
#
#
#    def renderUnitLabels (self, font, unit, color, indent):
#        """Help method for rendering one single label of an unit. Renders a lot Returns a tuple of
#        label. with (label, indent).""" 
#
#        return ( indent, ( font.render ( unit.getName (), color, (0,0,0)),
#                           font.render ( unit.getType (), color, (0,0,0)),
#                           font.render ( str (unit.getMen ()), color, (0,0,0)),
#                           font.render ( str (unit.getGuns ()), color, (0,0,0)) ) )


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: select_scenario.py,v 1.1.1.1 2002/02/19 10:16:04 slmm Exp $
###############################################################################################

import pygame
from pygame.locals import *
import os, string
import re, sys
import scenario
import properties
import widget
import urllib
from dialog import *
from button import Button
from label import Label
from image import Image
from widget_manager   import WidgetManager
from scenario_manager import ServerScenarioManager
from info_scenario    import InfoScenario
from scenario_parser  import ScenarioParser

class SelectScenario(Dialog):
    """
    This class is used as a dialog for selecting a scenario among the available scenarios. It
    displays all scenarios in a nice list.
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 12 )

        # define a map with the scenario names to the filenames
        self.namemap = { }
        self.info = None

        # no scenario data yet
        self.lines = None
        
        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )
        
        # no selection widget yet, it is created later
        self.selection = None

 
    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( Label (self.titlefont, "Select a scenario", (10, 10), color=(255, 0, 0)))

        # start index for the labels
        x = 100
        y = 100
        
        # create a scenario manager if we don't already have one. This will read the scenarios from
        # the server
        if scenario.scenario_manager == None:
            scenario.scenario_manager = ServerScenarioManager ()
    
            # read in all scenario info:s
            scenario.scenario_manager.loadScenarioIndex ( properties.path_scenarios )

        # start from index 0
        index = 0
        
        # loop over all infos we got
        for info in scenario.scenario_manager.getScenarios ():
            # get the name and location
            name = info.getName ()
            location = info.getLocation ()

            # get the date and create a string
            (year, month, day, hour, minute) = info.getDate ()
            date = `year` + "." + `month` + "." + `day` + " " + `hour` + ":" + `minute`

            # create a label
            text = name + ", " + location + ", " + date

            # create a label
            label = Label (self.smallfont, text, (x, y), color=(255, 255, 255),
                           callbacks = {widget.MOUSEBUTTONUP : self.select } )

            # register it
            self.wm.register ( label )

            # make sure we know the filename from the label later
            self.namemap [text] = ( index, info )
           
            # increment the y-offset and index
            y += label.getHeight () + 5
            index += 1

        # buttons. We create only the Cancel button so far, the other are created later
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (650, 650 ), {widget.MOUSEBUTTONUP : self.cancel } ) )



    def select (self, trigger, event):
        """Callback triggered when the user clicks one of the scenario labels. Shows the yellow box
        that indicates the wanted scenario as well as the 'Ok' button (if not shown)."""

        # get the url and index of the clicked label
        (index, info) = self.namemap [trigger.getText ()]

        # store the current scenario info
        self.info = info
        
        # now we have a selected scenario, show 'Ok' button and the 'Info' button
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (406, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )

        self.wm.register ( Button ( properties.path_dialogs + "butt-info-moff.png",
                                    properties.path_dialogs + "butt-info-mover.png",
                                    (162, 650 ), {widget.MOUSEBUTTONUP : self.showInfo } ) )

        
        # set coordinates for the box
        x = 70
        y = index * 22 + 98

        # do we have a selection widget?
        if self.selection == None:
            # nope, create and register a new image for the selection box
            self.selection = Image ( properties.scenario_selection_icon, (x,y) )
            self.wm.register ( self.selection )

        else:
            # already there, just reposition it
            self.selection.setPosition ( ( x, y ) )

        # we must repaint the entire dialog to get the old box cleared out
        self.wm.paint ( clear=1, force=1 )
        
        # we've selected a scenario
        self.state = ACCEPTED

        # we're done here
        return widget.HANDLED

    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Reads in a selected scenario."""

        # load the scenario
        self.lines = scenario.scenario_manager.loadScenario ( self.info.getId () )

        # did we get anything?
        if self.lines == None:
            # oops, bad data
            Messagebox ( "Could not load scenario '%s'!" % self.info.getName () )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
 
            # we're cancelling the dialog
            self.state = REJECTED
            return widget.DONE
 
        # parse the read data after flattening it to a string
        ScenarioParser ().parseString ( string.join ( self.lines ) )

        # we're ok the dialog
        self.state = ACCEPTED
        return widget.DONE


    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE
   

    def showInfo (self, trigger, event):
        """Callback triggered when the user clicks the 'Info' button. """

        # create a new dialog and run it
        InfoScenario ( self.info ).run ()
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)

        return widget.HANDLED


    def getChosenScenario (self):
        """Returns the data for the chosen scenario. The data is returned as a list of string. If no
        scenario had been chosen None is returned."""

        return self.lines
    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



from board import *

# Checks if the move is legal
def isMoveLegal(board, selector_pos):
	return len(board.state[selector_pos]) < 6 and selector_pos >= 0

def isBoardFull(board):
	for pos in range(7):
		if len(board.state[pos]) < 6:
			return 0 
	return 1

def isWinner(board, player):
        if _isVerticalWin(board, player):
                return 4
        if _isHorizontalWin(board, player):
                return 4
        if _isDiagonalWin(board, player):
                return 4
        return 0


def _isVerticalWin(board, player):
        x = board.last_move
        four_in_a_row = [player, player, player, player]
        return board.state[x][-4:] == four_in_a_row


def _isHorizontalWin(board, player):
        x = board.last_move
        y = len(board.state[x]) - 1
        four_in_a_row = str(player) * 4
        row = []
        for i in range(-3, 4):
                try:
                        if x+i < 0: continue
                        row.append(str(board.state[x+i][y]))
                except IndexError:
                        row.append('s')  # 's' stands for sentinel
        return ''.join(row).find(four_in_a_row) >= 0


def _isDiagonalWin(board, player):
        x = board.last_move
        y = len(board.state[x]) - 1
        four_in_a_row = str(player) * 4
        row = []
        for i in range(-3, 4):
                try:
                        if x+i < 0: continue
                        if y+i < 0: continue
                        row.append(str(board.state[x+i][y+i]))
                except IndexError:
                        row.append('s')  # 's' stands for sentinel
        if ''.join(row).find(four_in_a_row) >= 0:
                return 1
        row = []
        for i in range(-3, 4):
                try:
                        if x+i < 0: continue
                        if y-i < 0: continue
                        row.append(str(board.state[x+i][y-i]))
                except IndexError:
                        row.append('s')  # 's' stands for sentinel
        if ''.join(row).find(four_in_a_row) >= 0:
                return 1
        return 0

"""

# Checks if theres a winner
def isWinner(board, player):

	# Check vertical wins
	for x in range(7):
		sequence = 0
		for y in range(len(board.state[x])):
			if board.state[x][y] == player:
				sequence += 1
				if sequence == 4:
					# Whe've got a winner!!
					return sequence
			else: sequence = 0
	# Check horizontal wins
	for y in range(6):
		sequence = 0
		for x in range(7):
			# Make sure we wont go out of bounds
			if len(board.state[x]) > y:
				if board.state[x][y] == player:
					sequence += 1
                	                if sequence == 4:
                        	               # Whe've got a winner!!
                                	       return sequence
				else: sequence = 0
			else: sequence = 0

	# Check diagonally - bottom to top
	for height in range(6+1-4):
		for start in range(7+1-4):
			sequence = 0
			for pos in range(7 - start):
				# Make sure we wont go out of bounds
	                        if len(board.state[pos+start]) > (pos + height):
				 	if board.state[pos+start][pos+ height] == player:
						sequence += 1
						if sequence == 4:
							# Whe've got a winner!!
							return sequence
					else: sequence = 0
				else: sequence = 0

	# Check diagonally - top to bottom
	for height in range(6+2-4):
		for start in range(7+1-4):
			sequence = 0
			for pos in range(7-start):
				# Make sure we wont go out of bounds
	                        if (len(board.state[pos+start]) > (6 - pos - height) )  and (6 - pos - height) >=0 :
				 	if board.state[pos+start][6 - pos - height] == player:
						sequence += 1
						if sequence == 4:
							# Whe've got a winner!!
							return sequence
					else: sequence = 0
				else: sequence = 0
	# No win for player
	return 0
"""


###############################################################################################
# $Id: dialog.py,v 1.3 2002/02/26 15:35:56 slmm Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.event
import pygame.time
from pygame.locals import *
import widget
from widget import *
from button import Button
from label import Label
from widget_manager import WidgetManager
#import messagebox

# return codes used to indicate what should happen
ACCEPTED = 0
REJECTED = 1

class Dialog(Widget):
    """
    This class is used as a baseclass for dialogs. A dialog is a window with widgets that occupies
    the whole or part of the screen. Currently only whole screen dialogs are supported. This class
    contains the main widget manager to which widgets are added. Subclasses could override the
    method createWidgets() to create their own widgets, but that is not mandatory.

    It also contains the main event loop which is started using the method run(). The event loop
    handles events from all registered widgets and calls their callbacks.

    A dialog can have a background. It is set using setBackground() and can be either painted once
    or tiled over the entire screen.
    """

    # define a shared cache
    cache = {}

    def __init__ (self, surface):
        "Creates the dialog."
        # call superclass
        Widget.__init__ (self)
        
        # create the widget manager so that it manages our canvas and then paints out its stuff on
        # the root window
        self.wm = WidgetManager (surface, self)

	self.surface = surface

        # register ourselves as a paintable object to the widget manager. This should insure that we
        # get repainted too
        self.wm.register ( self )
        
        # create all widgets
        self.createWidgets ()

        # flags used for indicating what we've done
        self.state = REJECTED

        # no background
        self.background = None
        self.tile = 0

        
    def createWidgets (self):
        "Creates all widgets for the dialog. Override in subclasses."
        pass


    def setBackground (self, image, tile=0):
        """Sets a background for the entire dialog. The background is loaded from the passed
        'filename'. It is painted either only once or tiled, depending on the setting for 'tile'. A
        value of 1 will tile the image, any other value will draw the background once centered."""

        self.background = image 
            
        # store the tiling value
        self.tile = tile

        # we're dirty now
        self.dirty = 1
        

    def getWidth (self):
        "Returns the width of the dialog."
        return 0


    def getHeight (self):
        "Returns the height of the widget."
        return 0


    def getGeometry (self):
        """Returns the geometry of the widget. This is a tuple containing the x, y, width and height
        of the dialog. Not currently meaningful."""
        return ( self.position [0], self.position [1], 0, 0 )


    def isInside (self,position):
        """Checks wether the passed point is inside the dialog. Returns 1 if inside and 0 if
        outside. A point on the border of the widget is considered to be inside. Currently always
        returns 0."""
        # not inside
        return 0


    def paint (self, destination, force=0):
        """Method that paints the dialog. This method will simply blit out the background if one has
        been set. Override if custom painting is needed."""
        # are we dirty or not?
        if self.background == None or ( self.dirty == 0 and force == 0 ):
            # not dirty, or no background nothing to do here
            return 0

        # get the dimensions of the background
        width, height = self.background.get_size () 
        
        # should we tile or not?
        if self.tile:
            # perform tiling of the background
            for y in range ( self.surface.get_height() / height + 1 ):
                for x in range ( self.surface.get_width() / width + 1 ):
                    # what height should be used?
                    if y * height > self.surface.get_height:
                        # use only the missing part
                        heighty = (( y + 1 ) * height) - self.surface.get_height()

                    else:
                        # full height of icon
                        heighty = height

                    # what width should be used?
                    if x * width > self.surface.get_width():
                        # use only the missing part
                        widthx = (( x + 1 ) * width) - self.surface.get_width()

                    else:
                        # full width of icon
                        widthx = width

                        # blit it all out
                        destination.blit ( self.background, (x * width, y * height ) )
                        
        else:
            # no tiling, just blurt it out once
            destination.blit ( self.background, self.position )

        self.dirty = 0
        
        # we did something, make sure the widget manager knows that
        return 1



    def run (self):
        """Executes the dialog and runs its internal loop until a callback returns widget.DONE. When
        that dialog is terminated this method also returns.""" 

        # loop forever
        while 1:
            # repaint the stuff if needed
            self.wm.paint ()

            # get next event
            event = pygame.event.wait()

            # see wether the widget manager wants to handle it
            if event != -1:
                # handle event and get the return code that tells us wehter it was handled or not
                returncode = self.wm.handle ( event )

                # is the event loop done?
                if returncode == widget.DONE:
                    # disable the timer
                    self.disableTimer ()

                    return self.state

    def handleEvent(self, event):
		# repaint the stuff if needed
 		self.wm.paint ()

		# see wether the widget manager wants to handle it
		if event != -1:
                	# handle event and get the return code that tells us wehter it was handled or not
                	returncode = self.wm.handle ( event )

			# is the event loop done?
			if returncode == widget.DONE:
				# disable the timer
				self.disableTimer ()

				return self.state
			else:
				return -1
		else:
			return -1


    def accept (self):
        """Accepts the dialog. Will close it and return from it's event loop with the return status
        'dialog.ACCEPTED'."""
        # we're accepting the dialog
        self.state = ACCEPTED

        return widget.DONE


    def reject (self, trigger, event):
        """Accepts the dialog. Will close it and return from it's event loop with the return status
        'dialog.REJECTED'."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def messagebox (self, message):
        
        # failed to init network connection, show a messagebox
        messagebox.Messagebox ( message )

        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


    def enableTimer (self, ms):
        """Enables timer events. Calling this method will make the method timer() get called every
        'ms' milliseconds. Call disableTimer() to disable the timer."""
        # just call the method, make no checks
        pygame.time.set_timer ( USEREVENT, ms )
            

    def disableTimer (self):
        """Disables timer events."""
        # just call the method, make no checks
        pygame.time.set_timer ( USEREVENT, 0 )

        # remove all old stale events that may have been left in the queue
        pygame.event.get ( [USEREVENT] )
   

    def timer (self):
        """Callback triggered when the dialog has enabled timers and a timer fires. This should be
        overridden by subclasses to provide the needed code."""
        # by default we're handled
        return widget.HANDLED
    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: fader.py,v 1.1.1.1 2002/02/19 10:15:59 slmm Exp $
###############################################################################################

from widget import Widget
import widget
import pygame
from pygame.locals import *

class Fader (Widget):
    """
    This class defines a simple surface that works as a fader widget. It covers a part of a dialog
    with asurface that can gradially become less and less transparent, thus fading the part of the
    dialog towards that color.

    Call the setAlpha() method periodically to alter alpha.
    """

    def __init__ (self, size=(0,0), position=(0,0), alpha=0, color=(0,0,0), callbacks=None):
        """Initializes the widget. Creates the surface."""

        # firts call superclass constructor
        Widget.__init__ (self, position, callbacks)

        # create the surface
        self.surface = pygame.Surface ( size )

        # set the alpha too
        self.surface.set_alpha ( alpha, RLEACCEL )

        # store alpha too
        self.alpha = alpha
        

    def getAlpha (self):
        """Returns current alpha value."""
        return self.alpha
    

    def setAlpha (self, alpha):
        """Sets a new alpha value for the fader."""
        # store new alpha
        self.alpha = alpha
        
        # set the alpha too
        self.surface.set_alpha ( alpha, RLEACCEL )

        # we're dirty now
        self.dirty = 1
        
       
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: widget.py,v 1.1.1.1 2002/02/19 10:16:02 slmm Exp $
###############################################################################################

import pygame
import pygame.image
from pygame.locals import *

# define two additional events
MOUSEENTEREVENT = 100
MOUSELEAVEEVENT = 101
MOUSEBUTTONDOWN = pygame.MOUSEBUTTONDOWN
MOUSEBUTTONUP   = pygame.MOUSEBUTTONUP
MOUSEMOTION     = pygame.MOUSEMOTION
KEYUP           = pygame.KEYUP
KEYDOWN         = pygame.KEYDOWN
QUIT            = pygame.QUIT

# define a few constants we need
HANDLED   = 0
UNHANDLED = 1
DONE      = 2

class Widget:
    """
    This class defines a baseclass for all GUI widgets. It contains some common operations such as
    size handling and paiting the widget. In itself this class is not useful, and should be
    subclassed by more specialized classes.

    Subclasses should provide a SDL surface (self.surface) that can be blitted out when painting the
    widget as well as a map of the events  the widget is interested in (self.events). The map
    is checked every time an event happens within the widget, and the assigned callback is called if
    one such callback is registered. By default no widget has any callbacks registered.

    slm added the args, they are used for all callbacks so just ignore them if you dont want em
    if u want to pass multiple then make it a tuple or dict or whatever
    """

    def __init__ (self, position = (0,0), callbacks = None, args = None):
        """Initializes the widget. Subclasses are required to set the member self.surface to
        something valid. This is the visible part of the widget."""

        # store the position
        self.position = position

        # no surface yet
        self.surface = None

        # set the default callbacks
        self.callbacks = callbacks

	self.args = args

        # set default internal callbacks. Those widgets who have internal callbacks should set them
        # manually 
        self.internal = None
        
        # we're dirty by default
        self.dirty = 1


    def getWidth (self):
        "Returns the width of the widget."
        return self.surface.et_width ()


    def getHeight (self):
        "Returns the height of the widget."
        return self.surface.get_height ()


    def getX (self):
        "Returns the x-position of the widget."
        return self.position [0]

 
    def getY (self):
        "Returns the y-position of the widget."
        return self.position [1]


    def getPosition (self):
        "Returns a position containing the x- and y-position of the widget."
        return self.position


    def setPosition (self, position):
        """Set a new position for the widget. The passed position must be a tuple ontaining the new x-
        and y-position of the widget."""
        self.position = position

        # we're dirty now
        self.dirty = 1


    def getGeometry (self):
        """Returns the geometry of the widget. This is a tuple containing the x, y, width and height
        of the widget"""
        return ( self.position [0], self.position [1],
                 self.surface.get_width (), self.surface.get_height () )


    def isInside (self,position):
        """Checks wether the passed point is inside the widget. Returns 1 if inside and 0 if
        outside. A point on the border of the widget is considered to be inside."""
        ourx = self.position [0]
        oury = self.position [1]
        width = self.surface.get_width ()
        height = self.surface.get_height ()
        x = position[0]
        y = position[1]
        
        # is it inside?
        if ourx <= x and x <= ourx + width and oury <= y and y <= oury + height:
            # it's inside
            return 1

        # not inside
        return 0


    def paint (self, destination, force=0):
        """Method that paints the widget. This method will simply blit out the surface of the widget
        onto the destination surface. Override if custom painting is needed."""
        # are we dirty or not?
        if not self.dirty and not force:
            # not dirty, nothing to do here
            return 0
        
        # we're dirty, blit out our full surface to the main surface
        destination.blit ( self.surface, (self.position [0], self.position [1] ) )

        self.dirty = 0
        
        # we did something, make sure the widget manager knows that
        return 1
   

    def handle (self, type, event):
        """Handles the passed event if a callback has been registered. If no callback for the event
        can be found then this method just returns."""
        # should it be handled internally?
        if self.internal and self.internal.has_key ( type ) and self.internal[type] != None:
            # yep, so execute it, but we don't care about any return value
            self.internal[type] ( event )
        
        # do we have an external handler for that type?
        if self.callbacks and self.callbacks.has_key ( type ) and self.callbacks[type] != None:	
	    # yep, so execute it and return whatever it returns
            if self.args is not None:
	    	code = self.callbacks[type] ( self, event, self.args )
            else:
	    	code = self.callbacks[type] ( self, event )

            # did we get a code at all?
            if code == None:
                # ah, nope, so return a default answer
                return HANDLED
            
            # we got a good answer, return it
            return code
        
        # the widget is not interested in the eent
        return UNHANDLED

    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: editfield.py,v 1.2 2002/04/06 12:00:00 slmm Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.surface
import pygame.key
from pygame.locals import *
from widget import Widget
import widget

class EditField(Widget):
    """
    This class defines a simple edit field where the user may input some text while the widget has
    focus. 
    """

    # static images for the frame around the editable text
    frameicons = {}

    # names for the icons
    frameiconnames = [ 'topleft', 'top', 'topright', 'right', 'botright', 'bot', 'botleft', 'left' ]

    
    def __init__ (self, font, text="", width=100, position = (0,0), cursor = None, frameicons = None, callbacks = None,
    	color = (255,255,255), background = (0,0,0)):		      
        """Initializes the widget. Renders the font using the passed data."""

        # firts call superclass constructor
        Widget.__init__ (self, position, callbacks)

        # store the text
        if text == None:
            self.text = ""
        else:
            self.text = text

        # store the cursor position
        self.cursor = len ( self.text )

        # store all needed data so that we can create new surfaces later
        self.font = font
        self.color = color
        self.background = background
        self.width = width
	self.cursor_image = cursor
       	EditField.frameicons = frameicons

       
        # render the text
        self.textrendered = font.render ( text, 1, color )
        
        # create the background surface to fit the rendered text's height
        self.textsurface = pygame.Surface ( ( width, self.textrendered.get_height () + 3),
                                            HWSURFACE ).convert ()

        # fill with the proper background color 
        self.textsurface.fill ( background ) 

         # create the surface with the border in it
        self.createBorder ()
        
        # set our internal callbacks so that we can trap keys
        self.internal = {widget.KEYDOWN : self.keyDown }


    def setText (self, text):
        """Sets a new text to be rendered."""
        # store new text
        if text == None:
            self.text = ""
        else:
            self.text = text

        # render a new surface
        self.textrendered = self.font.render ( self.text, 1, self.color )



    def getText (self):
        """Returns the text of the editfield. The text is never None, but may be empty if the
        editfield was cleared."""
        return self.text



    def paint (self, destination, force=0):
        """Method that paints the editfield. This method will simply blit out the surface of the widget
        onto the destination surface. """
        # are we dirty or not?
        if not self.dirty and not force:
            # not dirty, nothing to do here
            return 0

        # we're dirty, blit out the frame first
        destination.blit ( self.surface, (self.position [0], self.position [1]) )
        
        # now the text background
        destination.blit ( self.textsurface, (self.position [0] + self.deltax, self.position [1] + self.deltay) )

        # and the text if we have any
        if self.textrendered:
            # yep, we have it, get the width we should use
            if self.width < self.textrendered.get_size ()[0] + 3:
                width = self.width - 3
            else:
                width = self.textrendered.get_size ()[0]

            # now blit out the text
            destination.blit ( self.textrendered,
                               (self.position [0] + self.deltax + 3, self.position [1] + self.deltay+ 3) )
            
        self.dirty = 0
        

	# slm : render cursor
        if len(self.text):
                cursorx = self.font.size(self.text[0:self.cursor])[0]
                cursor_space = self.font.size(" ")[0] / 2

                destination.blit(self.cursor_image,
                        (
                                (self.position [0] + self.deltax)+(cursorx+cursor_space),
                                (self.position [1] + self.deltay+ 3)
                        )
                )

	
        # we did something, make sure the widget manager knows that
        return 1



    def keyDown (self,  event):
 	"""Callback triggered when the user presses a key inside the editfield. Gets the pressed key
        and modifies the internal string if needed. Renders a new surface too if needed. Some
        special keys are ignored."""

        # get the key and the unicode string
        key = event.key

        # Weird fix. This is required on RedHat pygame 1.5.3??
        ev_unicode = event.unicode
        if type(ev_unicode) == unicode or type(ev_unicode) == str:
            if ev_unicode == '':
                ev_unicode = 0
            else:
                ev_unicode = ord(ev_unicode)

        value = unichr(ev_unicode).encode('latin1')

        # a special key?
        if key == K_BACKSPACE:
            # remove one character
            self.text = self.text [0:self.cursor - 1]

            # update cursor position
            self.cursor -= 1
            if self.cursor < 0:
                self.cursor = 0

            # what text should we render?
            if len(self.text) == 0:
                # no text, so use something empty
                text = ' '
            else:
                # we have text, use it
                text = self.text

            # render the new surface
            self.textrendered = self.font.render ( text, 1, self.color )

            # we're dirty now
            self.dirty = 1
            return

        if key == K_DELETE:
            # remove one character from the right
            self.text = self.text [0:self.cursor] + self.text[self.cursor + 1:]

            # what text should we render?
            if len(self.text) == 0:
                # no text, so use something empty
                text = ' '
            else:
                # we have text, use it
                text = self.text

            # render the new surface
            self.textrendered = self.font.render ( text, 1, self.color )

            # we're dirty now
            self.dirty = 1
            return

        if key == K_TAB or key == K_RETURN:
            # remove one character from the right
            print "editfield.keyDown(): tab/return not handled"
            return

        # arrow handling
        elif key == K_LEFT:
            self.cursor -= 1
            if self.cursor < 0:
                self.cursor = 0

        elif key ==  K_RIGHT:
            self.cursor += 1
            if self.cursor > len(self.text):
                self.cursor = len(self.text)

        elif key == K_HOME:
            # move to the first position
            self.cursor = 0

        elif key == K_END:
            # move the the last position
            self.cursor = len(self.text)

        elif ev_unicode != 0:
            # a normal key pressed, merge in the text
            self.text = self.text [0:self.cursor] + value + self.text[self.cursor:]

            # update cursor position
            self.cursor += 1

            # do we still have any text left?
            if self.text == "":
                self.textrendered = None
                self.cursor = 0
            else:
                # set the new surface
                self.textrendered = self.font.render ( self.text, 1, self.color )

            # we're dirty now
            self.dirty = 1
	    return

    def createBorder (self):
        """Creates the border for the widget. Uses the 8 static icons and creates a surface that is
        slightly larger than the text surface (self.textsurface). This method is one ugly thing of
        coordinates and offsets. Don't touch unless you know what you do."""
        # get the delta values. These are used so that we know where to paint the text surface
        self.deltax = EditField.frameicons ['left'].get_width()
        self.deltay = EditField.frameicons ['top'].get_height()

        # get height and width of new surface
        size   = self.textsurface.get_size ()
        width  = size[0] + self.deltax + EditField.frameicons ['right'].get_width()
        height = size[1] + self.deltay + EditField.frameicons ['bot'].get_height()

        # create the needed surface
        self.surface = pygame.Surface ( (width, height), HWSURFACE ).convert ()

        # now some gory details. We need to blit stuff onto our frame surface to create the actual
        # frame. We first blit out the borders and the the corners. the corners then overpaint the
        # borders at proper positions
        top      = EditField.frameicons ['top']
        bot      = EditField.frameicons ['bot']
        left     = EditField.frameicons ['left']
        right    = EditField.frameicons ['right']
        topleft  = EditField.frameicons ['topleft']
        topright = EditField.frameicons ['topright']
        botleft  = EditField.frameicons ['botleft']
        botright = EditField.frameicons ['botright']

        # top/bottom borders
        x = 0
        while x < width:
            # what width should be used?
            if x + top.get_size ()[0] < width:
                # full width of icon
                widthx = top.get_size ()[0]
            else:
                # use only the missing part
                widthx = width - x

            # blit it all out
            self.surface.blit ( top, ( x, 0))
            self.surface.blit ( bot, ( x, height - bot.get_height () + 1 ) )
            x += widthx

        # left/right borders
        y = 0
        while y < height:
            # what height should be used?
            if y + left.get_size ()[1] < height:
                # full height of icon
                heighty = left.get_size ()[1]
            else:
                # use only the missing part
                heighty = height - y

            # blit it all out
            self.surface.blit ( left, ( 0, y ))
            self.surface.blit ( right, ( width - right.get_size ()[0], y ))
            y += heighty
        

        # top left corner
        self.surface.blit ( topleft, ( 0, 0 ))

        # top right corner
        self.surface.blit ( topright, ( width -  topright.get_width (), 0 ))

        # bottom left corner
        self.surface.blit ( botleft, ( 0, height - botleft.get_height () ))

        # bottom right corner
        self.surface.blit ( botright, ( width -  botright.get_width(), height - botright.get_height()))

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



# Created by Jeroen Vloothuis, 2002
import sys
from inisettings import *

def getOptions(argv):
	opts= {}
	while argv:
		if argv[0][0] == '-':
			opts[argv[0]] = argv[1]
			argv = argv[2:]
		else:
			argv = argv[1:]
	return opts


options = getOptions(sys.argv)

settings = IniSettings(options['-ini'])
for key in settings.settings.keys():
	for item in settings.settings[key].keys():
		settings.set(key,item, int(int(settings.settings[key][item])*float(options['-size'])))
settings.save()
